{
  This is a basic driver for the TI XX calculator series.  It handles the low level sending and receiving of
  bits over TI 3 wire D-Bus.  This version of the code has been tested to send bits to the calculator and can
  be made to send keyboard scan codes to the calculator with ease.
  Thanks to  Benjamin Ryves originally made this work for
  the PicAxe microcomputer.  http://www.ticalc.org/archives/files/fileinfo/366/36697.html
  This work is derived directly from his code.  

  This code is released free to the public.  Please note however that this driver is very low level and should
  work with most TI caculators, however it is a far cry from being actually useful by itself.
  Refer to the detailed documentation for the specific caculator you want to use.  You will still need to implement the
  Software link protocol (or some portion of it) which is calculator specific for this to acutally be useful.
  A huge amount of information has been reversed engineered for many TI calcs and can be found at
  http://www.ticalc.org/pub/text/calcinfo/
  Look for the download called tixx_guide.zip.

  --------------------------- THE HARDWARE ---------------
   
  The link cable is made up of 3 leads - two data lines and a common
  grounding. The cable is tipped with 2.5mm stereo jacks.
  If you cut up a TI calc-to-calc cable, you will see that one wire is
  coloured red and the other white. Red is D0 in these routines, white
  is D1. Red is the tip of the stereo jack plug. White is the ring. This
  connector is a standard "mini" stereo plug.  Radio Shack usually has the female
  connectors you can use to add a port to your propeller.  Just Run D0 and D1 to two
  pins on the prop and ground the other wire
'                                          ______
'                      __   ___   ________/     /
'                     /  \_/   \_/        |     \
'                    | D0 |  D1 | GROUND  |     / 
'                    |RED |WHITE| COPPER  |     \
'                     \__/ \___/ \________|     /
'                                         \_____\

  Initialize this object by calling start and passing in the pin numbers to use.
}
VAR
  byte D0
  byte D1  
  byte bRec
PUB start(ring, tip)
  D0 := tip
  D1 := ring  
  dira[D0] := 1
  dira[D1] := 1
  outa[D0] := 1
  outa[D1] := 1

  dira[D0] := 0
  dira[D1] := 0
  
PUB commReady        ' checks to see that both the ring and D0 are high.. this is the normal waiting state of the TI-92 comm lines
  result := True
  if ina[D1] == 0 OR  ina[D0] == 0
     Abort False

PUB sendTIBit1   ' sends a bit = 1 to the calculator
  commReady
  Return sendTIBit(D1,D0)
PUB sendTIBit0   ' sends a bit = 0 to the calculator
  commReady
  Return sendTIBit(D0,D1)

PUB sendByte(bSend) | index  ' sends a complete 8 bit byte to the calculator, use with an Abort Trap for error handling
  result := True
  repeat  index  from 0 to 7
    if bSend & %00000001 == %1
      
      if NOT  \sendTIBit1
        Return FALSE
    else
      if NOT \sendTIBit0
        Return FALSE
        
    bSend >>= 1
  'term.out(lf)
  
{
  Call this method after sending bytes to the calculator.  The TI sends acknowledgement data back
  after each command received (see software protocol documenation for more information)  this data must
  be recieved or the TI will wait for two seconds.  This method simply acknowledges the reciept of the
  bytes by handshaking but does nothing with them.  Perhaps some future versions will store the data in memory for later
  use by the propeller.
}
PUB  recData |index,sTime,bit, state   ' data comes MSB first
  result := true
  sTime := cnt
  dira[D1] := 0
  dira[D0] := 0

  ' wait for response from the TI for up to 2 sec
  repeat while ina[D1] + ina[D0] == 2 AND cnt - sTime < 10_000_000 
      waitcnt(10_000 + cnt)
  state := ina[D1] + ina[D0] 
  if state == 2  ' no response from the TI, so abort. 
    Abort False
   
  repeat  while ina[D1] + ina[D0] < 2
    bit := \recBit
    if bit == -1
      return
    else
      bRec := %10000000 & bRec
      bRec >>= 1
      waitcnt(1_000 + cnt)
    
    

{
  Sends a single bit to the TI using appropriate handshaking.
  Whether the bit is a 1 or 0 depends on whether DO or D1 is sent as the drive and ack pin.
  D0 as drive => 0
  D1 as drive => 1
  This works by setting the drive pin low, waiting for the ack to go low,
  then setting the drive pin high and waiting for ack to go high.
  That sends one bit.  The TI protocol specifies that timeout should occur if no ack is 
  felt in 2 seconds.  Timeouts result in an Abort False  
}
PUB sendTIBit(drive,ack) | sTime
  
  result := True
  sTime := cnt
  dira[ack] := 0
  dira[drive] := 1
  outa[drive] := 0                ' set the drive low to indicating a 1 or 0 bit
  ' Wait for the Ack bit to go low
  repeat while cnt - sTime < 10_000_000 AND ina[ack] == 1   ' wait for the Ack to go low (this indicates the calc got the bit) 
      waitCnt(10_000 + cnt)
  if ina[ack] == 1
    Abort False
  if ina[ack] == 0              ' Calc acknowledges the bit
    dira[drive] :=1
    outa[drive] := 1            ' set the drive line back to high (data has been recieved)
    repeat while cnt - sTime < 10_000_000 AND ina[ack] == 0  ' Wait for the Ack to go high
      waitCnt(10_000 + cnt)
    if ina[ack] == 0
      Abort False
    dira[drive] := 0
  else                          ' ack is still high, no response,  reset the drive line to high and abort
    dira[drive] :=1
    outa[drive] := 1            ' set the drive line back to high
    dira[drive] := 0
    Abort False

PUB recbit | b ,sTime

    
    if ina[D1] == 0 OR ina[D0] == 0
      if ina[D1] == 0   ' incoming 1 bit
        result := 1
        dira[D0] := 1
        outa[D0] := 0   ' indicates bit recieved
        sTime := cnt
        repeat while cnt - sTime < 5_000_000 AND ina[D1] == 0
          waitcnt( 10_000 + cnt )
        outa[D0] := 1
        if ina[D1] == 0
          Abort -1
        dira[D0] := 0
        
      if ina[D0] == 0    ' incoming 0 bit0 (take no action here because the left shit automatically gives us a 0
        result := 0
        dira[D1] := 1
        outa[D1] := 0   ' indicates bit recieved
        sTime := cnt
        repeat while cnt - sTime < 5_000_000 AND ina[D0] == 0
          waitcnt( 10_000 + cnt )
        outa[D1] := 1
        if ina[D0] == 0
          Abort -1
        dira[D1] := 0
       
    else
      Abort false
